#include <avr/io.h> 
#include <util/delay.h>

/****************************************************************
 * Define all consts
 ****************************************************************/
// CORE is running at 1.2Mhz (9.6Mhz with the DIV8 bit enabled)
// this is configured in AVR Studio project settings and required
// for delay.h functions
#define SENSOR_POWER_PIN PINB3
#define LED_PIN PINB2
#define SENSOR_TRIGGER_PIN PINB1
#define PHOTO_SENSOR PINB4 // not technically used, we set ADMUX but want to show it's used and ADC is set for PINB4
#define F_CPU 1200000  // 1.2 MHz

/**********************************************************************
 * Method to enable ADC, read value of pin, disable ADC, return value
 * Assumptions: ADC_init() has been called to cofigure ADC
 *				Always want to sample 8 times and average
 *				Always enable and disable ADC with each read
 * Input: NONE
 **********************************************************************/
int ADC_read(void)
{
	int i;
	int ADC_temp;
	
	int ADCr = 0;
	
	ADCSRA = (1<<ADEN); // enable the ADC
	
	//do a dummy readout first
	ADCSRA |= (1<<ADSC); // do single conversion
	
	// wait for conversion done, ADIF flag active
	while(!(ADCSRA & 0x10));
	
	// do the ADC conversion 8 times for better accuracy
	for(i=0;i<8;i++)
	{
		ADCSRA |= (1<<ADSC); // do single conversion
		// wait for conversion done, ADIF flag active
		while(!(ADCSRA & 0x10));

		ADC_temp = ADCL; // read out ADCL register
	
		ADC_temp += (ADCH << 8); // read out ADCH register
	
		// accumulate result (8 samples) for later averaging
		ADCr += ADC_temp;
	}

	ADCr = ADCr >> 3; // average the 8 samples
	ADCSRA = (0<<ADEN); // disable the ADC

	return ADCr;
}

/****************************************************************
 * Method to initalize the ADC on the ATTINY13A
 * Assumptions: ADC readings will only be on PINB3
 *				AVR will have a freq of 1.2Mhz
 *				Will use VCC as reference
 * Input: NONE
 ****************************************************************/
void ADC_init(void){
	int tempvar =0;
	ADMUX = (1<<MUX1); // set PB4 as input
	ADCSRA = (1<<ADPS1)| (1<<ADPS0); // set adc to divide by 8 1.2Mhz / 8 = 150khz NOTE: may need to set this to 16 to esure we are below the spec limit 4 resolution
	tempvar = ADC_read(); // read once to clear out noise/etc
}

/****************************************************************
 * Method to call _delay_ms multiple times to ensure a proper long delay
 * Assumptions: Clock is set correctly and delay.h is included
 * Input: int delay in seconds
 ****************************************************************/
void delay_seconds(int Seconds){
	int i = 0;
	int timesToLoop;
	timesToLoop = (Seconds * 10);
	for(i=0;i<timesToLoop;i++){
		_delay_ms(100);
	}
}

/****************************************************************
 * Method to blink the LED on and off pausing 200ms between
 * Assumptions: All LED output pins will be on PORTB
 * Input: int blinks: number of times to blink LED
 * 		  int ledPin: the output pin number to use for LED
 ****************************************************************/
void blinkLEDs(int blinks,int ledPin){
	int t;
	for(t=0;t<blinks;t++){
		PORTB |= (1<<ledPin);
		_delay_ms(200);
		PORTB &= ~(1<<ledPin);
		_delay_ms(200);
	}
}

int main(void){

	/****************************************************************
	 * Modify DDR and I/O settings
	 ****************************************************************/
	// enable leds, sensor power for output (all others as input)
	DDRB = (1<<LED_PIN) | (1<<SENSOR_POWER_PIN);

	// turn off all outputs (low voltage), enable pull-up resistor on sensor pin
	PORTB = (0<<LED_PIN) | (0<<SENSOR_POWER_PIN) | (1<<SENSOR_TRIGGER_PIN);



	/****************************************************************
	 * declare all func vars
	 ****************************************************************/
	int adcValue = 0; 			// tmp var to hold our value from adc
	int ToggleCounter = 0;		// var to hold the count up or down for darkness/light before flipping PIR system on/off
	int ToggleOn = 0;			// toggle to know if PIR should be up and running or shut down

	// initalize ADC
	ADC_init();

	// loop forever
 	while(1){
		delay_seconds(1); // delay one second per loop
		
		adcValue = ADC_read(); // read photocell
		//adcValue = 100;

		if(adcValue < 80){
			// dark out
			if(ToggleCounter == 5) ToggleOn = 1;
			else ToggleCounter++;
		}else{
			// light out
			if(ToggleCounter == 0) ToggleOn = 0;
			else ToggleCounter--;
		}

		if(ToggleOn == 1){
				// if power isn't enbled for PIR do so, then check for motion
				if(bit_is_clear(PORTB,SENSOR_POWER_PIN)){
					blinkLEDs(1,LED_PIN); 			// blink LEDs once to show us sensor is ready to calibrate
					PORTB |= (1<<SENSOR_POWER_PIN);  
					delay_seconds(30); 				// delay for 30 seconds to allow sensor to calibrate (per datasheet)
					blinkLEDs(3,LED_PIN);			// blink LEDs three times to show the PIR is alive and checking for motion
				}
				// power is on already, check for motion
				if(bit_is_set(PINB,SENSOR_TRIGGER_PIN)){
					//motion detected, light up leds for 12 seconds
					PORTB |= (1<<LED_PIN);
					delay_seconds(12);
					PORTB &= ~(1<<LED_PIN);
				} 
		}else{
			// must be light out, and we reached our toggle limit, shut down 
			PORTB &= ~(1<<SENSOR_POWER_PIN);
		}
	}
}
